@@ -20,15 +20,17 @@
       also be specified individually, e.g. if the docs should be buildable
       without Internet access.
 
-    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 from __future__ import print_function
 
+import sys
 import time
 import zlib
 import codecs
+import functools
 import posixpath
 from os import path
 import re
@@ -68,7 +70,7 @@ def read_inventory_v1(f, uri, join):
     return invdata
 
 
-def read_inventory_v2(f, uri, join, bufsize=16*1024):
+def read_inventory_v2(f, uri, join, bufsize=16 * 1024):
     invdata = {}
     line = f.readline()
     projname = line.rstrip()[11:].decode('utf-8')
@@ -91,7 +93,7 @@ def read_inventory_v2(f, uri, join, bufsize=16*1024):
             lineend = buf.find(b'\n')
             while lineend != -1:
                 yield buf[:lineend].decode('utf-8')
-                buf = buf[lineend+1:]
+                buf = buf[lineend + 1:]
                 lineend = buf.find(b'\n')
         assert not buf
 
@@ -116,7 +118,7 @@ def read_inventory_v2(f, uri, join, bufsize=16*1024):
     return invdata
 
 
-def read_inventory(f, uri, join, bufsize=16*1024):
+def read_inventory(f, uri, join, bufsize=16 * 1024):
     line = f.readline().rstrip().decode('utf-8')
     if line == '# Sphinx inventory version 1':
         return read_inventory_v1(f, uri, join)
@@ -164,6 +166,9 @@ def _read_from_url(url, config=None):
     r = requests.get(url, stream=True, config=config, timeout=config.intersphinx_timeout)
     r.raise_for_status()
     r.raw.url = r.url
+    # decode content-body based on the header.
+    # ref: https://github.com/kennethreitz/requests/issues/2155
+    r.raw.read = functools.partial(r.raw.read, decode_content=True)
     return r.raw
 
 
@@ -221,8 +226,8 @@ def fetch_inventory(app, uri, inv):
             try:
                 join = localuri and path.join or posixpath.join
                 invdata = read_inventory(f, uri, join)
-            except ValueError:
-                raise ValueError('unknown or unsupported inventory version')
+            except ValueError as exc:
+                raise ValueError('unknown or unsupported inventory version: %r' % exc)
     except Exception as err:
         app.warn('intersphinx inventory %r not readable due to '
                  '%s: %s' % (inv, err.__class__.__name__, err))
@@ -242,7 +247,7 @@ def load_mappings(app):
     cache = env.intersphinx_cache
     update = False
     for key, value in iteritems(app.config.intersphinx_mapping):
-        if isinstance(value, tuple):
+        if isinstance(value, (list, tuple)):
             # new format
             name, (uri, inv) = key, value
             if not isinstance(name, string_types):
@@ -317,6 +322,9 @@ def missing_reference(app, env, node, contnode):
         if not objtypes:
             return
         objtypes = ['%s:%s' % (domain, objtype) for objtype in objtypes]
+    if 'std:cmdoption' in objtypes:
+        # until Sphinx-1.6, cmdoptions are stored as std:option
+        objtypes.append('std:option')
     to_try = [(env.intersphinx_inventory, target)]
     in_set = None
     if ':' in target:
@@ -342,9 +350,9 @@ def missing_reference(app, env, node, contnode):
                     (domain == 'std' and node['reftype'] == 'keyword'):
                 # use whatever title was given, but strip prefix
                 title = contnode.astext()
-                if in_set and title.startswith(in_set+':'):
-                    newnode.append(contnode.__class__(title[len(in_set)+1:],
-                                                      title[len(in_set)+1:]))
+                if in_set and title.startswith(in_set + ':'):
+                    newnode.append(contnode.__class__(title[len(in_set) + 1:],
+                                                      title[len(in_set) + 1:]))
                 else:
                     newnode.append(contnode)
             else:
@@ -366,17 +374,26 @@ def setup(app):
     return {'version': sphinx.__display_version__, 'parallel_read_safe': True}
 
 
-if __name__ == '__main__':
-    # debug functionality to print out an inventory
-    import sys
+def debug(argv):
+    """Debug functionality to print out an inventory"""
+    if len(argv) < 2:
+        print("Print out an inventory file.\n"
+              "Error: must specify local path or URL to an inventory file.",
+              file=sys.stderr)
+        sys.exit(1)
+
+    class MockConfig(object):
+        intersphinx_timeout = None  # type: int
+        tls_verify = False
 
     class MockApp(object):
         srcdir = ''
+        config = MockConfig()
 
         def warn(self, msg):
             print(msg, file=sys.stderr)
 
-    filename = sys.argv[1]
+    filename = argv[1]
     invdata = fetch_inventory(MockApp(), '', filename)
     for key in sorted(invdata or {}):
         print(key)
@@ -384,3 +401,7 @@ if __name__ == '__main__':
             print('\t%-40s %s%s' % (entry,
                                     einfo[3] != '-' and '%-40s: ' % einfo[3] or '',
                                     einfo[2]))
+
+
+if __name__ == '__main__':
+    debug(argv=sys.argv)
