@@ -6,7 +6,7 @@
     "Doc fields" are reST field lists in object descriptions that will
     be domain-specifically transformed to a more appealing presentation.
 
-    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 from __future__ import absolute_import
@@ -55,35 +55,37 @@ class Field(object):
         self.bodyrolename = bodyrolename
 
     def make_xref(self, rolename, domain, target,
-                  innernode=addnodes.literal_emphasis, contnode=None):
+                  innernode=addnodes.literal_emphasis, contnode=None, env=None):
         if not rolename:
             return contnode or innernode(target, target)
         refnode = addnodes.pending_xref('', refdomain=domain, refexplicit=False,
                                         reftype=rolename, reftarget=target)
         refnode += contnode or innernode(target, target)
+        if env:
+            env.domains[domain].process_field_xref(refnode)
         return refnode
 
     def make_xrefs(self, rolename, domain, target,
-                   innernode=addnodes.literal_emphasis, contnode=None):
-        return [self.make_xref(rolename, domain, target, innernode, contnode)]
+                   innernode=addnodes.literal_emphasis, contnode=None, env=None):
+        return [self.make_xref(rolename, domain, target, innernode, contnode, env)]
 
     def make_entry(self, fieldarg, content):
         return (fieldarg, content)
 
-    def make_field(self, types, domain, item):
+    def make_field(self, types, domain, item, env=None):
         fieldarg, content = item
         fieldname = nodes.field_name('', self.label)
         if fieldarg:
             fieldname += nodes.Text(' ')
             fieldname.extend(self.make_xrefs(self.rolename, domain,
-                                             fieldarg, nodes.Text))
+                                             fieldarg, nodes.Text, env=env))
 
         if len(content) == 1 and (
                 isinstance(content[0], nodes.Text) or
                 (isinstance(content[0], nodes.inline) and len(content[0]) == 1 and
                  isinstance(content[0][0], nodes.Text))):
             content = self.make_xrefs(self.bodyrolename, domain,
-                                      content[0].astext(), contnode=content[0])
+                                      content[0].astext(), contnode=content[0], env=env)
         fieldbody = nodes.field_body('', nodes.paragraph('', '', *content))
         return nodes.field('', fieldname, fieldbody)
 
@@ -109,13 +111,13 @@ class GroupedField(Field):
         Field.__init__(self, name, names, label, True, rolename)
         self.can_collapse = can_collapse
 
-    def make_field(self, types, domain, items):
+    def make_field(self, types, domain, items, env=None):
         fieldname = nodes.field_name('', self.label)
         listnode = self.list_type()
         for fieldarg, content in items:
             par = nodes.paragraph()
             par.extend(self.make_xrefs(self.rolename, domain, fieldarg,
-                                       addnodes.literal_strong))
+                                       addnodes.literal_strong, env=env))
             par += nodes.Text(' -- ')
             par += content
             listnode += nodes.list_item('', par)
@@ -155,11 +157,11 @@ class TypedField(GroupedField):
         self.typenames = typenames
         self.typerolename = typerolename
 
-    def make_field(self, types, domain, items):
+    def make_field(self, types, domain, items, env=None):
         def handle_item(fieldarg, content):
             par = nodes.paragraph()
             par.extend(self.make_xrefs(self.rolename, domain, fieldarg,
-                                       addnodes.literal_strong))
+                                       addnodes.literal_strong, env=env))
             if fieldarg in types:
                 par += nodes.Text(' (')
                 # NOTE: using .pop() here to prevent a single type node to be
@@ -169,7 +171,7 @@ class TypedField(GroupedField):
                 if len(fieldtype) == 1 and isinstance(fieldtype[0], nodes.Text):
                     typename = u''.join(n.astext() for n in fieldtype)
                     par.extend(self.make_xrefs(self.typerolename, domain, typename,
-                                               addnodes.literal_emphasis))
+                                               addnodes.literal_emphasis, env=env))
                 else:
                     par += fieldtype
                 par += nodes.Text(')')
@@ -196,7 +198,7 @@ class DocFieldTransformer(object):
     """
 
     def __init__(self, directive):
-        self.domain = directive.domain
+        self.directive = directive
         if '_doc_field_type_map' not in directive.__class__.__dict__:
             directive.__class__._doc_field_type_map = \
                 self.preprocess_fieldtypes(directive.__class__.doc_field_types)
@@ -308,7 +310,7 @@ class DocFieldTransformer(object):
             else:
                 fieldtype, content = entry
                 fieldtypes = types.get(fieldtype.name, {})
-                new_list += fieldtype.make_field(fieldtypes, self.domain,
-                                                 content)
+                new_list += fieldtype.make_field(fieldtypes, self.directive.domain,
+                                                 content, env=self.directive.env)
 
         node.replace_self(new_list)
