@@ -5,49 +5,65 @@
 
     Simple requests package loader
 
-    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 from __future__ import absolute_import
 
-import requests
 import warnings
+from contextlib import contextmanager
+
+import requests
 import pkg_resources
 
 from six import string_types
 from six.moves.urllib.parse import urlsplit
 try:
-    from requests.packages.urllib3.exceptions import SSLError, InsecureRequestWarning
+    from requests.packages.urllib3.exceptions import SSLError
 except ImportError:
     # python-requests package in Debian jessie does not provide ``requests.packages.urllib3``.
     # So try to import the exceptions from urllib3 package.
-    from urllib3.exceptions import SSLError, InsecureRequestWarning
+    from urllib3.exceptions import SSLError
 
-# try to load requests[security]
 try:
-    pkg_resources.require(['requests[security]'])
-except (pkg_resources.DistributionNotFound,
-        pkg_resources.VersionConflict):
+    from requests.packages.urllib3.exceptions import InsecureRequestWarning
+except ImportError:
+    try:
+        # for Debian-jessie
+        from urllib3.exceptions import InsecureRequestWarning
+    except ImportError:
+        # for requests < 2.4.0
+        InsecureRequestWarning = None
+
+# try to load requests[security] (but only if SSL is available)
+try:
     import ssl
-    if not getattr(ssl, 'HAS_SNI', False):
-        # don't complain on each url processed about the SSL issue
-        requests.packages.urllib3.disable_warnings(
-            requests.packages.urllib3.exceptions.InsecurePlatformWarning)
+except ImportError:
+    pass
+else:
+    try:
+        pkg_resources.require(['requests[security]'])
+    except (pkg_resources.DistributionNotFound,
+            pkg_resources.VersionConflict):
+        if not getattr(ssl, 'HAS_SNI', False):
+            # don't complain on each url processed about the SSL issue
+            requests.packages.urllib3.disable_warnings(
+                requests.packages.urllib3.exceptions.InsecurePlatformWarning)
+            warnings.warn(
+                'Some links may return broken results due to being unable to '
+                'check the Server Name Indication (SNI) in the returned SSL cert '
+                'against the hostname in the url requested. Recommended to '
+                'install "requests[security]" as a dependency or upgrade to '
+                'a python version with SNI support (Python 3 and Python 2.7.9+).'
+            )
+    except pkg_resources.UnknownExtra:
         warnings.warn(
             'Some links may return broken results due to being unable to '
             'check the Server Name Indication (SNI) in the returned SSL cert '
             'against the hostname in the url requested. Recommended to '
-            'install "requests[security]" as a dependency or upgrade to '
-            'a python version with SNI support (Python 3 and Python 2.7.9+).'
+            'install requests-2.4.1+.'
         )
-except pkg_resources.UnknownExtra:
-    warnings.warn(
-        'Some links may return broken results due to being unable to '
-        'check the Server Name Indication (SNI) in the returned SSL cert '
-        'against the hostname in the url requested. Recommended to '
-        'install requests-2.4.1+.'
-    )
 
 useragent_header = [('User-Agent',
                      'Mozilla/5.0 (X11; Linux x86_64; rv:25.0) Gecko/20100101 Firefox/25.0')]
@@ -65,6 +81,15 @@ def is_ssl_error(exc):
             return False
 
 
+@contextmanager
+def ignore_insecure_warning(**kwargs):
+    with warnings.catch_warnings():
+        if not kwargs.get('verify') and InsecureRequestWarning:
+            # ignore InsecureRequestWarning if verify=False
+            warnings.filterwarnings("ignore", category=InsecureRequestWarning)
+        yield
+
+
 def _get_tls_cacert(url, config):
     """Get addiotinal CA cert for a specific URL.
 
@@ -96,10 +121,7 @@ def get(url, **kwargs):
     if config:
         kwargs.setdefault('verify', _get_tls_cacert(url, config))
 
-    with warnings.catch_warnings():
-        if not kwargs.get('verify'):
-            # ignore InsecureRequestWarning if verify=False
-            warnings.filterwarnings("ignore", category=InsecureRequestWarning)
+    with ignore_insecure_warning(**kwargs):
         return requests.get(url, **kwargs)
 
 
@@ -112,8 +134,5 @@ def head(url, **kwargs):
     if config:
         kwargs.setdefault('verify', _get_tls_cacert(url, config))
 
-    with warnings.catch_warnings():
-        if not kwargs.get('verify'):
-            # ignore InsecureRequestWarning if verify=False
-            warnings.filterwarnings("ignore", category=InsecureRequestWarning)
+    with ignore_insecure_warning(**kwargs):
         return requests.get(url, **kwargs)
